package nl.iobyte.themeparkconnector;

import com.google.gson.Gson;
import net.milkbowl.vault.economy.Economy;
import nl.iobyte.themeparkconnector.api.ThemeParkConnectorAPI;
import nl.iobyte.themeparkconnector.api.config.enums.StorageKey;
import nl.iobyte.themeparkconnector.commands.ThemeParkConnectorCommand;
import nl.iobyte.themeparkconnector.listeners.*;
import nl.iobyte.themeparkconnector.sbd.License;
import nl.iobyte.themeparkconnector.sbd.UpdateManager;
import org.bukkit.Bukkit;
import org.bukkit.plugin.PluginManager;
import org.bukkit.plugin.RegisteredServiceProvider;
import org.bukkit.plugin.java.JavaPlugin;

public class ThemeParkConnector extends JavaPlugin {

    private final static Gson gson = ThemeParkConnectorGson.getInstance();

    private static ThemeParkConnector instance;
    private final ThemeParkConnectorAPI api = new ThemeParkConnectorAPI();
    private Economy econ = null;
    private boolean disabling = false;

    public void onEnable() {
        if (!setupEconomy() ) {
            Bukkit.getLogger().info(String.format("[%s] - No Vault dependency found!", getDescription().getName()));
        }

        instance = this;
        api.enable();

        //License stuff
        new License(this, "TPP", api.getConfigurationManager().getString(StorageKey.LICENSE));

        loadCommands();
        loadListeners();
        loadUpdateManager();
    }

    private void loadUpdateManager() {
        if(!api.getConfigurationManager().getBoolean(StorageKey.UPDATE_CHECK_ENABLED))
            return;

        UpdateManager manager = new UpdateManager(
                this,
                5,
                api.getConfigurationManager().getString(StorageKey.LICENSE)
        );
        manager.handleResponse((response, version) -> {
            switch (response) {
                case LATEST:
                    getLogger().info("You're running the latest version("+version.get()+")");
                    break;
                case THIS_NEWER:
                    getLogger().info("You're running a newer version("+version.get()+")");
                    break;
                case UNAVAILABLE:
                    getLogger().info("Unable to check for updates");
                    break;
                case FOUND_NEW:
                    getLogger().info("Found newer version("+version.get()+")");
                    if(!api.getConfigurationManager().getBoolean(StorageKey.UPDATE_DOWNLOAD_ENABLED))
                        return;

                    manager.handleDownloadResponse((downloadResponse, fileName) -> {
                        switch (downloadResponse) {
                            case DONE:
                                getLogger().info("Update downloaded! If you restart your server, it will be loaded. Filename: " + fileName);
                                break;
                            case ERROR:
                                getLogger().severe("Something went wrong when trying downloading the latest version.");
                                break;
                            case UNAVAILABLE:
                                getLogger().warning("Unable to download the latest version.");
                                break;
                        }
                    }).runUpdate();
            }
        }).check();
    }

    private boolean setupEconomy() {
        if(getServer().getPluginManager().getPlugin("Vault") == null)
            return false;

        RegisteredServiceProvider<Economy> rsp = getServer().getServicesManager().getRegistration(Economy.class);
        if (rsp == null)
            return false;

        econ = rsp.getProvider();
        return econ != null;
    }

    private void loadCommands() {
        new ThemeParkConnectorCommand();
    }

    private void loadListeners() {
        PluginManager pm = Bukkit.getPluginManager();
        pm.registerEvents(new PlayerListener(), this);
        pm.registerEvents(new OperatorListener(), this);
        pm.registerEvents(new AttractionListener(), this);
        pm.registerEvents(new RideCountListener(), this);
        pm.registerEvents(new TicketListener(), this);
        pm.registerEvents(new RideOperatorListener(), this);
    }

    public void onDisable() {
        disabling = true;
        api.disable();

        instance = null;
    }

    public static Gson getGson() {
        return gson;
    }

    public static ThemeParkConnector getInstance() {
        return instance;
    }

    public Economy getEconomy() {
        return econ;
    }

    public ThemeParkConnectorAPI getAPI() {
        return api;
    }

    public boolean isDisabling() {
        return disabling;
    }
}
